function [minRampDist, minFbaDist] = QPdist(expData, graphName)
%
% requires expData = an m x 3 array of m experimental fluxes
%   the first col is the flux indices of the iJO1366b model associated with
%      the experimental data
%   the second col is the flux data
%   the third col is is the eperimental range
%

% add cobra and gurobi's path
addpath(genpath('/opt/cobra/'));
addpath(genpath('/opt/gurobi605/linux64/matlab'));

% settings
numScenarios = 5;
mu = 0;
sigma = 1;
numSigma = 3;
delta = 3;
scaleP = 0.2;
scaleGrwth = 0.9;
tol = 10^(-8);

% solver settings
params.outputflag = 0;
%params.Method = 2; %-1 automatic, 0 primal Simplex, 1 dual simplex, 2 barrier
%params.NumericFocus = 3;

% load and solve the model and identify the growth coefficients
load modeldata; % student model

model = modeldata.iJO1366b;
%
% Remove O2 if desired
%
if ~isempty(strfind(graphName, '2'))
    model.lb(252) = 0; 
end

%
% Solve the FBA problem
%
[m,n] = size(model.S);
gmodel.A = sparse(model.S);
gmodel.obj = model.c;
gmodel.rhs = zeros(m,1);
gmodel.lb = model.lb;
gmodel.ub = model.ub;
gmodel.modelsense = 'max';
for i = 1:m
   gmodel.sense(i) = '=';
end
gsoln = gurobi(gmodel, params);
grwthVal = gsoln.objval;

% Generate a probability vector
probVec = zeros(numScenarios,1);  % instantiate space for probability vector
stepSize = 2*sigma*numSigma/numScenarios; % divide +/- numSigma std. dev. into appropriate widths
currentIndex = mu - numSigma*sigma;  % initiate the counter to the most left value
for i=1:ceil(numScenarios/2)
    p = normcdf([currentIndex, currentIndex + stepSize],mu,sigma); % calculate cumulative prob.
    probVec(i) = p(2) - p(1); % discretize the piece we want
    probVec(numScenarios-i+1) = probVec(i); % set its mirror image on the bottom side
    currentIndex = currentIndex + stepSize; % index the step
end
% the calculation above misses the tails of the normal, we distribute this error over the scenarios
probVec = probVec + ((1-sum(probVec))/numScenarios);

% Find the growth equation
grwCol = find(model.c == 1);
grwRow = find(model.S(:,grwCol) ~= 0);

% Build the opt. model's coefficient matrix
S1 = model.S;
S2 = model.S;
for coefInd = 1:length(grwRow)
   grwInd = grwRow(coefInd);
   
   % Generate scenarios
   model.S(grwInd, grwCol);
   scenMat = repmat(model.S(grwInd,grwCol), [numScenarios,1]);
   scenMat = scenMat(:) - scaleP*[-1;-1/2;0;1/2;1].*scenMat(:);
   matAdj = norm(delta*sqrt(diag(probVec))*(eye(numScenarios) - ones(numScenarios,1)*probVec')*scenMat);
      
   % Build model with adjusted coefficients
   S1(grwInd, grwCol) = S1(grwInd, grwCol) + matAdj;
   S2(grwInd, grwCol) = S2(grwInd, grwCol) - matAdj;
end
      
% Calculate M
eM = zeros(m,1);
eM(grwRow) = 1;
eM = diag(eM); % use the identity to minimize the 1-norm
eM = eM(:, find(sum(eM) == 1));
[~, numColAdd] = size(eM);
clear gmodelTmp;
gmodelTmp.A = sparse([-S1, eM; S2, eM; [model.c', zeros(1,numColAdd)]]);
gmodelTmp.rhs = [zeros(2*m,1); grwthVal];
gmodelTmp.obj = [zeros(n,1); ones(numColAdd,1)];
gmodelTmp.lb = [model.lb; zeros(numColAdd,1)];
gmodelTmp.ub = [model.ub; inf*ones(numColAdd,1)];
gmodelTmp.modelsense = 'min';
for i = 1:2*m+1
   gmodelTmp.sense(i) = '>';
end
gsolnTmp = gurobi(gmodelTmp,params);
% make sure there are no pesky negatives
M = max(10^(-6),gsolnTmp.x(n+1:end)); 

% Build the Robust Model      
clear gmodelTmp;
gmodelTmp.A = sparse([-S1; S2; model.c']);
b = zeros(m,1);
b(grwRow) = -M;
gmodelTmp.rhs = [b;b;scaleGrwth*grwthVal];
dQ = zeros(n,1);
dQ(expData(:,1)) = 1;
gmodelTmp.Q = sparse(diag(dQ));
gmodelTmp.obj(expData(:,1)) = -2*expData(:,2);
gmodelTmp.lb = model.lb;
gmodelTmp.ub = model.ub;
gmodelTmp.modelsense = 'min';
for i = 1:2*m+1
   gmodelTmp.sense(i) = '>';
end
% set some parameters, the interior point algorithms surpisingly
% do well here
params.BarIterLimit = 2000;
params.BarConvTol = 10^(-4);
params.FeasibilityTol = 10^(-6);
params.OptimalityTol = 10^(-6);
params.Quad = 1;
params.outputflag = 0;
params.BarHomogeneous = 1;
% grab the solution
gsolnTmp = gurobi(gmodelTmp,params);
minRampDist = gsolnTmp.objval + norm(expData(:,2))^2;

% solve the FBA model
fbaQPModel.A = sparse([model.S; model.c']);
fbaQPModel.rhs = [model.b; scaleGrwth*grwthVal] ;
fbaQPModel.obj = gmodelTmp.obj;
fbaQPModel.Q = gmodelTmp.Q;
fbaQPModel.lb = model.lb;
fbaQPModel.ub = model.ub;
fbaQPModel.modelsense = 'min';
for i = 1:m
   fbaQPModel.sense(i) = '=';
end
fbaQPModel.sense(m+1) = '>';
fbaQPSoln = gurobi(fbaQPModel, params);
minFbaDist = fbaQPSoln.objval + norm(expData(:,2))^2;

% check flux 2081
negInd = find(expData(:,1) == 2081, 1);
if ~isempty(negInd)
   if expData(negInd,2) < 0
      expData(negInd,2) = -expData(negInd,2);
      fbaQPSoln.x(2081) = -fbaQPSoln.x(2081);
      gsolnTmp.x(2081) = -gsolnTmp.x(2081);
   end
end

% comment this out if images are desired
%return

close all;
fig=figure;
hold on;
indx = [1:length(expData(:,2))]';
plot(indx, gsolnTmp.x(expData(:,1)), 'r*', 'linewidth', 2)
plot(indx, fbaQPSoln.x(expData(:,1)), 'b*', 'linewidth', 2)
plot(indx, expData(:,2), 'ko', 'MarkerSize', 10, 'linewidth', 2)
for k = 1:length(indx)
   z = linspace(expData(k,2)-expData(k,3), expData(k,2)+expData(k,3), 40);
   plot(k*ones(1,length(z)), z, 'k:', 'linewidth', 2)
end

eval(sprintf('saveas(fig, ''%s'', ''png'')', graphName))
